"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var plugin_exports = {};
__export(plugin_exports, {
  default: () => plugin_default
});
module.exports = __toCommonJS(plugin_exports);
var path = __toESM(require("path"));
var import_express = __toESM(require("express"));
var import_cookie_parser = __toESM(require("cookie-parser"));
var import_utils = require("@modern-js/utils");
var import_finalhandler = __toESM(require("finalhandler"));
var import_node = require("@modern-js/server-core/base/node");
var import_context = require("./context");
var import_registerRoutes = __toESM(require("./registerRoutes"));
const findAppModule = async (apiDir) => {
  const exts = [
    ".ts",
    ".js"
  ];
  const paths = exts.map((ext) => path.resolve(apiDir, `app${ext}`));
  for (const filename of paths) {
    if (await import_utils.fs.pathExists(filename)) {
      delete require.cache[filename];
      return [
        (0, import_utils.compatRequire)(filename),
        require(filename)
      ];
    }
  }
  return [];
};
const initMiddlewares = (middleware, app) => {
  middleware.forEach((middlewareItem) => {
    const middlewareFunc = typeof middlewareItem === "string" ? (0, import_utils.compatRequire)(middlewareItem) : middlewareItem;
    app.use(middlewareFunc);
  });
};
const useRun = (app) => {
  app.use((req, res, next) => {
    (0, import_context.run)({
      req,
      res
    }, next);
  });
};
const createApp = async ({ apiDir, middlewares, mode, apiHandlerInfos, render }) => {
  let app;
  if (mode === "framework") {
    const appModule = await findAppModule(apiDir);
    app = appModule[0];
    const hooks = appModule[1];
    if (!(app === null || app === void 0 ? void 0 : app.use)) {
      app = (0, import_express.default)();
    }
    initApp(app);
    if (middlewares && middlewares.length > 0) {
      initMiddlewares(middlewares, app);
    }
    useRun(app);
    (0, import_registerRoutes.default)(app, apiHandlerInfos);
    if (hooks) {
      const { afterLambdaRegisted } = hooks;
      if (afterLambdaRegisted) {
        afterLambdaRegisted(app);
      }
    }
  } else if (mode === "function") {
    app = (0, import_express.default)();
    initApp(app);
    if (middlewares && middlewares.length > 0) {
      initMiddlewares(middlewares, app);
    }
    useRun(app);
    (0, import_registerRoutes.default)(app, apiHandlerInfos);
  } else {
    throw new Error(`mode must be function or framework`);
  }
  if (render) {
    app.use(async (req, res, next) => {
      const response = await render(req.__honoRequest, {
        logger: import_utils.logger,
        nodeReq: req,
        templates: req.__templates,
        serverManifest: req.__serverManifest
      });
      if (response) {
        return (0, import_node.sendResponse)(response, res).then(next);
      }
      next();
    });
  }
  return app;
};
const initApp = (app) => {
  app.use((0, import_cookie_parser.default)());
  app.use(import_express.default.text());
  app.use(import_express.default.urlencoded({
    extended: true
  }));
  app.use(import_express.default.json());
  return app;
};
var plugin_default = () => {
  let app;
  let apiDir;
  let mode;
  let renderHtml;
  return {
    name: "@modern-js/plugin-express",
    pre: [
      "@modern-js/plugin-bff"
    ],
    post: [
      "@modern-js/plugin-server"
    ],
    setup: (api) => ({
      async onApiChange(changes) {
        const appContext = api.useAppContext();
        const middlewares = appContext.apiMiddlewares;
        const apiHandlerInfos = appContext.apiHandlerInfos;
        app = await createApp({
          apiDir,
          middlewares,
          mode,
          apiHandlerInfos,
          render: renderHtml
        });
        return changes;
      },
      async prepareApiServer({ pwd, render }) {
        var _userConfig_bff;
        const appContext = api.useAppContext();
        const apiHandlerInfos = appContext.apiHandlerInfos;
        const { apiDirectory } = appContext;
        const userConfig = api.useConfigContext();
        const middlewares = appContext.apiMiddlewares;
        mode = appContext.apiMode;
        renderHtml = ((_userConfig_bff = userConfig.bff) === null || _userConfig_bff === void 0 ? void 0 : _userConfig_bff.enableHandleWeb) && render ? render : void 0;
        apiDir = apiDirectory || path.join(pwd, "./api");
        app = await createApp({
          apiDir,
          middlewares,
          mode,
          apiHandlerInfos,
          render: renderHtml
        });
        const handler = (req, res) => new Promise((resolve, reject) => {
          const handler2 = (err) => {
            if (err) {
              return reject(err);
            }
            (0, import_finalhandler.default)(req, res, {})(null);
            return resolve();
          };
          res.on("finish", (err) => {
            if (err) {
              return reject(err);
            }
            return resolve();
          });
          return app(req, res, handler2);
        });
        return (0, import_node.httpCallBack2HonoMid)(handler);
      }
    })
  };
};
